//Defines
#define ID_EDIT           10001
#define ID_STATUS         10002
#define ID_TAB            10003
#define ID_FIRSTMDI       10010

#define PD_UNLOAD               0
#define PD_NONUNLOAD_ACTIVE     1
#define PD_NONUNLOAD_NONACTIVE  2

#define EDL_FAILED                 0
#define EDL_UNLOADED               1
#define EDL_NONUNLOADED_ACTIVE     2
#define EDL_NONUNLOADED_NONACTIVE  3

#define AUTOANSWER_ASK    0
#define AUTOANSWER_YES    1
#define AUTOANSWER_NO     2

#define OD_ADT_BINARY_ERROR      0x00000001
#define OD_ADT_REG_CODEPAGE      0x00000002
#define OD_ADT_DETECT_CODEPAGE   0x00000004
#define OD_ADT_DETECT_BOM        0x00000008

#define EOD_ADT_OPEN        -1
#define EOD_ADT_ALLOC       -2
#define EOD_ADT_READ        -3
#define EOD_ADT_BINARY      -4
#define EOD_OPEN            -5
#define EOD_CANCEL          -6
#define EOD_WINDOW_EXIST    -7
#define EOD_CODEPAGE_ERROR  -8
#define EOD_STOP            -9
#define EOD_STREAMIN        -10

#define ESD_OPEN            -1
#define ESD_WRITE           -2
#define ESD_READONLY        -3
#define ESD_CODEPAGE_ERROR  -4
#define ESD_STOP            -5
#define ESD_STREAMOUT       -6

#define SS_REGISTRY  1
#define SS_INI       2

#define NEWLINE_WIN  1
#define NEWLINE_UNIX 2

#define PO_READ      1
#define PO_SAVE      2

#define PO_DWORD     1
#define PO_BINARY    2
#define PO_STRING    3

#define NCM_EDIT     1
#define NCM_TAB      2
#define NCM_STATUS   3

#ifndef FR_DOWN
  #define FR_DOWN      0x00000001
#endif
#ifndef FR_MATCHCASE
  #define FR_MATCHCASE 0x00000004
#endif
#define FR_UP          0x00100000
#define FR_BEGINNING   0x00200000
#define FR_SELECTION   0x00400000
#define FR_ESCAPESEQ   0x00800000
#define FR_ALLFILES    0x01000000

#define IMENU_EDIT     0x00000001
#define IMENU_CHECKS   0x00000004


//Structures
typedef void (CALLBACK *PLUGINPROC)(void *);
typedef void (CALLBACK *WNDPROCRET)(CWPRETSTRUCT *);

typedef struct _OPENDOCUMENTA {
  char szFile[MAX_PATH];       //File to open
  char szWorkDir[MAX_PATH];    //Set working directory before open, if (!*szWorkDir) then don't set
  DWORD dwFlags;               //open flags
                               //OD_ADT_BINARY_ERROR     Check if file is binary
                               //OD_ADT_REG_CODEPAGE     If last open code page found in registry, then it will be
                               //                        used with ADT_DETECT_BOM flag, if not found, then next flags
                               //                        will be used ADT_DETECT_CODEPAGE|ADT_DETECT_BOM
                               //OD_ADT_DETECT_CODEPAGE  Detect code page
                               //OD_ADT_DETECT_BOM       Detect BOM mark
  int nCodePage;               //File code page, ignored if (dwFlags & ADT_DETECT_CODEPAGE)
  BOOL bBOM;                   //File BOM, ignored if (dwFlags & ADT_DETECT_BOM)
} OPENDOCUMENTA;

typedef struct _OPENDOCUMENTW {
  wchar_t wszFile[MAX_PATH];     //File to open
  wchar_t wszWorkDir[MAX_PATH];  //Set working directory before open, if (!*wszWorkDir) then don't set
  DWORD dwFlags;                 //open flags
                                 //OD_ADT_BINARY_ERROR     Check if file is binary
                                 //OD_ADT_REG_CODEPAGE     If last open code page found in registry, then it will be
                                 //                        used with ADT_DETECT_BOM flag, if not found, then next flags
                                 //                        will be used ADT_DETECT_CODEPAGE|ADT_DETECT_BOM
                                 //OD_ADT_DETECT_CODEPAGE  Detect code page
                                 //OD_ADT_DETECT_BOM       Detect BOM mark
  int nCodePage;                 //File code page, ignored if (dwFlags & ADT_DETECT_CODEPAGE)
  BOOL bBOM;                     //File BOM, ignored if (dwFlags & ADT_DETECT_BOM)
} OPENDOCUMENTW;

typedef struct _SAVEDOCUMENTA {
  char szFile[MAX_PATH];      //File to save
  int nCodePage;              //File code page
  BOOL bBOM;                  //File BOM
  BOOL bActivate;             //File will be activated
} SAVEDOCUMENTA;

typedef struct _SAVEDOCUMENTW {
  wchar_t wszFile[MAX_PATH];  //File to save
  int nCodePage;              //File code page
  BOOL bBOM;                  //File BOM
  BOOL bActivate;             //File will be activated
} SAVEDOCUMENTW;

typedef struct _WNDFRAMEA {
  HICON hIcon;                //Frame icon
  char szFile[MAX_PATH];      //Frame file
  int nCodePage;              //Frame edit code page
  BOOL bBOM;                  //Frame edit BOM
  int nNewLine;               //Frame edit new line
  BOOL bModified;             //Frame edit modified
  BOOL bReadOnly;             //Frame edit read only
  BOOL bWordWrap;             //Frame edit word wrap
  BOOL bInsertState;          //Frame edit insert state
  int nTabStopSize;           //Frame edit tab stop size
  int nUndoLimit;             //Frame edit undo limit
  BOOL bShowURL;              //Frame edit show URL
  DWORD dwEditMargins;        //Frame edit margins
  BOOL bDelimitersEnable;     //Frame edit delimiters enabled
  COLORREF crFont;            //Frame edit font color
  COLORREF crBackground;      //Frame edit background color
  FILETIME ft;                //Frame file time
  LOGFONTA lf;                //Frame edit font
} WNDFRAMEA;

typedef struct _WNDFRAMEW {
  HICON hIcon;                //Frame icon
  wchar_t wszFile[MAX_PATH];  //Frame file
  int nCodePage;              //Frame edit code page
  BOOL bBOM;                  //Frame edit BOM
  int nNewLine;               //Frame edit new line
  BOOL bModified;             //Frame edit modified
  BOOL bReadOnly;             //Frame edit read only
  BOOL bWordWrap;             //Frame edit word wrap
  BOOL bInsertState;          //Frame edit insert state
  int nTabStopSize;           //Frame edit tab stop size
  int nUndoLimit;             //Frame edit undo limit
  BOOL bShowURL;              //Frame edit show URL
  DWORD dwEditMargins;        //Frame edit margins
  BOOL bDelimitersEnable;     //Frame edit delimiters enabled
  COLORREF crFont;            //Frame edit font color
  COLORREF crBackground;      //Frame edit background color
  FILETIME ft;                //Frame file time
  LOGFONTW lf;                //Frame edit font
} WNDFRAMEW;

typedef struct _WNDPROCDATA {
  struct _WNDPROCDATA *next;
  struct _WNDPROCDATA *prev;
  WNDPROC CurProc;
  WNDPROC NextProc;
  WNDPROC PrevProc;
} WNDPROCDATA;

typedef struct _WNDPROCRETDATA {
  struct _WNDPROCRETDATA *next;
  struct _WNDPROCRETDATA *prev;
  WNDPROCRET CurProc;
  WNDPROCRET NextProc;
  WNDPROCRET PrevProc;
} WNDPROCRETDATA;

typedef struct _PLUGINDATA {
  DWORD cb;                 //Size of the structure
  int *nUnload;             //PD_UNLOAD               unload plugin (default)
                            //PD_NONUNLOAD_ACTIVE     don't unload plugin and set active status
                            //PD_NONUNLOAD_NONACTIVE  don't unload plugin and set non-active status
  BOOL bActive;             //Plugin already loaded
  BOOL bOnStart;            //TRUE  if plugin called on start-up
                            //FALSE if plugin called manually
  LPARAM lParam;            //Input data
  HINSTANCE hInstanceDLL;   //DLL instance
  HINSTANCE hInstanceEXE;   //EXE instance
  HWND hMainWnd;            //Main window
  HWND hWndEdit;            //RichEdit window
  HWND hStatus;             //StatusBar window
  HWND hMdiClient;          //MDI client window (if bMDI == TRUE)
  HWND hTab;                //Tab window        (if bMDI == TRUE)
  HMENU hMainMenu;          //Main menu
  HMENU hMenuRecentFiles;   //Recent files menu
  HMENU hMenuLanguage;      //Language menu
  HMENU hPopupMenu;         //Right click menu
  HICON hMainIcon;          //Main window icon handle
  BOOL bOldWindows;         //Non-Unicode Windows
  BOOL bOldRichEdit;        //riched20.dll lower then 5.30 (v3.0)
  BOOL bOldComctl32;        //comctl32.dll lower then 4.71
  BOOL bMDI;                //MDI mode
  unsigned char *pFile;     //Current editing file
                            //char *pFile         if bOldWindows == TRUE
                            //wchar_t *pFile      if bOldWindows == FALSE
  unsigned char *pAkelDir;  //AkelPad directory
                            //char *pAkelDir      if bOldWindows == TRUE
                            //wchar_t *pAkelDir   if bOldWindows == FALSE
  unsigned char *pFunction; //Called function name, format "Plugin::Function"
                            //char *pFunction     if bOldWindows == TRUE
                            //wchar_t *pFunction  if bOldWindows == FALSE
  int nSaveSettings;        //Setting in SS_REGISTRY or SS_INI
  LANGID wLangSystem;       //System language ID
} PLUGINDATA;

typedef struct _PLUGINFUNCTIONA {
  struct _PLUGINFUNCTIONA *next;
  struct _PLUGINFUNCTIONA *prev;
  char szFunction[MAX_PATH];      //Function name, format "Plugin::Function"
  int nFunctionLen;               //Function name length
  WORD wHotkey;                   //Function hotkey
  BOOL bOnStart;                  //Function autoload on start-up
  BOOL bRunning;                  //Function is running
  PLUGINPROC PluginProc;          //Function procedure
  void *lpParameter;              //Procedure parameter
} PLUGINFUNCTIONA;

typedef struct _PLUGINFUNCTIONW {
  struct _PLUGINFUNCTIONW *next;
  struct _PLUGINFUNCTIONW *prev;
  wchar_t wszFunction[MAX_PATH];  //Function name, format L"Plugin::Function"
  int nFunctionLen;               //Function name length
  WORD wHotkey;                   //Function hotkey
  BOOL bOnStart;                  //Function autoload on start-up
  BOOL bRunning;                  //Function is running
  PLUGINPROC PluginProc;          //Function procedure
  void *lpParameter;              //Procedure parameter
} PLUGINFUNCTIONW;

typedef struct _PLUGINCALLSENDA {
  char *pFunction;                //Function name, format "Plugin::Function"
  BOOL bOnStart;                  //TRUE  if plugin called on start-up
                                  //FALSE if plugin called manually
  LPARAM lParam;                  //Input data
} PLUGINCALLSENDA;

typedef struct _PLUGINCALLSENDW {
  wchar_t *wpFunction;            //Function name, format L"Plugin::Function"
  BOOL bOnStart;                  //TRUE  if plugin called on start-up
                                  //FALSE if plugin called manually
  LPARAM lParam;                  //Input data
} PLUGINCALLSENDW;

typedef struct _PLUGINCALLPOSTA {
  char szFunction[MAX_PATH];      //Function name, format "Plugin::Function"
  BOOL bOnStart;                  //TRUE  if plugin called on start-up
                                  //FALSE if plugin called manually
  LPARAM lParam;                  //Input data
} PLUGINCALLPOSTA;

typedef struct _PLUGINCALLPOSTW {
  wchar_t wszFunction[MAX_PATH];  //Function name, format L"Plugin::Function"
  BOOL bOnStart;                  //TRUE  if plugin called on start-up
                                  //FALSE if plugin called manually
  LPARAM lParam;                  //Input data
} PLUGINCALLPOSTW;

typedef struct PLUGINUNLOADSENDA {
  char *pFunction;                //Function name, format "Plugin::Function"
  HMODULE hInstanceDLL;           //NULL if plugin must not be unloaded.
  HANDLE hThread;                 //NULL if plugin not exiting from thread.
} PLUGINUNLOADSENDA;

typedef struct PLUGINUNLOADSENDW {
  wchar_t *wpFunction;            //Function name, format L"Plugin::Function"
  HMODULE hInstanceDLL;           //NULL if plugin must not be unloaded.
  HANDLE hThread;                 //NULL if plugin not exiting from thread.
} PLUGINUNLOADSENDW;

typedef struct PLUGINUNLOADPOSTA {
  char szFunction[MAX_PATH];      //Function name, format "Plugin::Function"
  HMODULE hInstanceDLL;           //NULL if plugin must not be unloaded.
  HANDLE hThread;                 //NULL if plugin not exiting from thread.
} PLUGINUNLOADPOSTA;

typedef struct PLUGINUNLOADPOSTW {
  wchar_t wszFunction[MAX_PATH];  //Function name, format L"Plugin::Function"
  HMODULE hInstanceDLL;           //NULL if plugin must not be unloaded.
  HANDLE hThread;                 //NULL if plugin not exiting from thread.
} PLUGINUNLOADPOSTW;

typedef struct _PLUGINOPTIONA {
  char *szOptionName;            //Option name
  DWORD dwType;                  //Data type: PO_DWORD, PO_BINARY or PO_STRING
  BYTE *lpData;                  //Data pointer
  DWORD dwData;                  //Data size in bytes
} PLUGINOPTIONA;

typedef struct _PLUGINOPTIONW {
  wchar_t *wszOptionName;        //Option name
  DWORD dwType;                  //Data type: PO_DWORD, PO_BINARY or PO_STRING
  BYTE *lpData;                  //Data pointer
  DWORD dwData;                  //Data size in bytes
} PLUGINOPTIONW;

typedef struct _GETTEXTRANGE {
  int cpMin;                      //First character in the range. First char of text: 0.
  int cpMax;                      //Last character in the range. Last char of text: -1.
  unsigned char *pText;           //Pointer to buffer that receives the text.
                                  //char *pText      if bOldWindows == TRUE
                                  //wchar_t *pText   if bOldWindows == FALSE
} GETTEXTRANGE;

typedef struct _EDITINFO {
  HWND hWndEdit;            //RichEdit window
  unsigned char *pFile;     //Current editing file
                            //char *pFile         if bOldWindows == TRUE
                            //wchar_t *pFile      if bOldWindows == FALSE
  int nCodePage;            //Current code page
  BOOL bBOM;                //Current BOM
  int nNewLine;             //Current new line format
  BOOL bModified;           //File has been modified
  BOOL bReadOnly;           //Read only
  BOOL bWordWrap;           //Word wrap
  BOOL bInsertState;        //Insert mode
  int nTabStopSize;         //Tab stop size
  BOOL bTabStopAsSpaces;    //Insert tab stop as spaces
  int nUndoLimit;           //Undo limit
  BOOL bDetailedUndo;       //Detailed undo
  BOOL bShowURL;            //Show URL
  DWORD dwEditMargins;      //Edit margins
  BOOL bDelimitersEnable;   //Delimiters enabled
  COLORREF crFont;          //Font color
  COLORREF crBackground;    //Background color
  FILETIME ft;              //File time
} EDITINFO;

typedef struct _RECENTFILESA {
  char (*lpszRecentNames)[MAX_PATH];      //Recent files names
  DWORD *lpdwRecentPositions;             //Recent files positions
  DWORD *lpdwRecentCodepages;             //Recent files codepages
  int nRecentFiles;                       //Number of recent files
} RECENTFILESA;

typedef struct _RECENTFILESW {
  wchar_t (*lpwszRecentNames)[MAX_PATH];  //Recent files names
  DWORD *lpdwRecentPositions;             //Recent files positions
  DWORD *lpdwRecentCodepages;             //Recent files codepages
  int nRecentFiles;                       //Number of recent files
} RECENTFILESW;

typedef struct _TEXTFINDA {
  DWORD dwFlags;            //Search flags
  char *pFindIt;            //Find string
} TEXTFINDA;

typedef struct _TEXTFINDW {
  DWORD dwFlags;            //Search flags
  wchar_t *wpFindIt;        //Find string
} TEXTFINDW;

typedef struct _TEXTREPLACEA {
  DWORD dwFlags;            //Search flags
  char *pFindIt;            //Find string
  char *pReplaceWith;       //Replace string
  BOOL bAll;                //Replace all
  int nChanges;             //Count of changes
} TEXTREPLACEA;

typedef struct _TEXTREPLACEW {
  DWORD dwFlags;            //Search flags
  wchar_t *wpFindIt;        //Find string
  wchar_t *wpReplaceWith;   //Replace string
  BOOL bAll;                //Replace all
  int nChanges;             //Count of changes
} TEXTREPLACEW;

typedef struct _TEXTRECODE {
  int nCodePageFrom;        //Source code page
  int nCodePageTo;          //Target code page
} TEXTRECODE;

typedef struct _NOPENDOCUMENTA {
  char *szFile;                  //Pointer to a file string buffer
  int *nCodePage;                //Pointer to a code page variable
  BOOL *bBOM;                    //Pointer to a BOM variable
  DWORD *dwFlags;                //Pointer to a open flags variable
  BOOL bProcess;                 //TRUE   open file
                                 //FALSE  do not open file
} NOPENDOCUMENTA;

typedef struct _NOPENDOCUMENTW {
  wchar_t *wszFile;              //Pointer to a file string buffer
  int *nCodePage;                //Pointer to a code page variable
  BOOL *bBOM;                    //Pointer to a BOM variable
  DWORD *dwFlags;                //Pointer to a open flags variable
  BOOL bProcess;                 //TRUE   open file
                                 //FALSE  do not open file
} NOPENDOCUMENTW;

typedef struct _NSAVEDOCUMENTA {
  char *szFile;                  //Pointer to a file string buffer
  int *nCodePage;                //Pointer to a code page variable
  BOOL *bBOM;                    //Pointer to a BOM variable
  BOOL bProcess;                 //TRUE   save file
                                 //FALSE  do not save file
} NSAVEDOCUMENTA;

typedef struct _NSAVEDOCUMENTW {
  wchar_t *wszFile;              //Pointer to a file string buffer
  int *nCodePage;                //Pointer to a code page variable
  BOOL *bBOM;                    //Pointer to a BOM variable
  BOOL bProcess;                 //TRUE   save file
                                 //FALSE  do not save file
} NSAVEDOCUMENTW;

typedef struct _NMAINSHOW {
  DWORD *dwStyle;            //Pointer to a maximized state variable (WS_MAXIMIZE or zero)
  DWORD *dwShow;             //Pointer to a SW_ constants combination variable
  BOOL bProcess;             //TRUE   show main window
                             //FALSE  do not show main window
} NMAINSHOW;

typedef struct _NCONTEXTMENU {
  HWND hWnd;                   //Context menu window
  UINT uType;                  //Type:    NCM_EDIT, NCM_TAB or NCM_STATUS
  POINT pt;                    //Context menu coordiates
  BOOL bProcess;               //TRUE   show context menu
                               //FALSE  do not show context menu
} NCONTEXTMENU;


//AkelPad menu messages:
#define IDM_FILE_NEW                    4101  //Create new RichEdit window
#define IDM_FILE_CREATENEW              4102  //Create new instance of program
#define IDM_FILE_OPEN                   4103  //Open file dialog
#define IDM_FILE_REOPEN                 4104  //Reopen file
#define IDM_FILE_SAVE                   4105  //Save file
#define IDM_FILE_SAVEAS                 4106  //Save file dialog
#define IDM_FILE_PAGESETUP              4107  //Print setup dialog
#define IDM_FILE_PRINT                  4108  //Print dialog
#define IDM_FILE_EXIT                   4109  //Exit program
#define IDM_FILE_SAVEALL                4110  //Save all (MDI)
#define IDM_EDIT_UNDO                   4151  //Undo last operation
#define IDM_EDIT_REDO                   4152  //Redo last operation
#define IDM_EDIT_CUT                    4153  //Cut
#define IDM_EDIT_COPY                   4154  //Copy
#define IDM_EDIT_PASTE                  4155  //Paste
#define IDM_EDIT_CLEAR                  4156  //Delete
#define IDM_EDIT_SELECTALL              4157  //Select all
#define IDM_EDIT_FIND                   4158  //Find dialog
#define IDM_EDIT_FINDNEXTDOWN           4159  //Find last string down
#define IDM_EDIT_FINDNEXTUP             4160  //Find last string up
#define IDM_EDIT_REPLACE                4161  //Replace dialog
#define IDM_EDIT_GOTOLINE               4162  //Go to line dialog
#define IDM_EDIT_INSERTCHAR             4163  //Run charmap.exe
#define IDM_EDIT_INSERT_TAB             4164  //Insert tabulation
#define IDM_EDIT_INSERT_TAB_MENU        4165  //Insert tabulation only if several lines selected
#define IDM_EDIT_DELETE_TAB             4166  //Delete tabulation
#define IDM_EDIT_DELETE_TAB_MENU        4167  //Delete tabulation only if several lines selected
#define IDM_EDIT_INSERT_SPACE           4168  //Insert space
#define IDM_EDIT_INSERT_SPACE_MENU      4169  //Insert space only if several lines selected
#define IDM_EDIT_DELETE_SPACE           4170  //Delete space
#define IDM_EDIT_DELETE_SPACE_MENU      4171  //Delete space only if several lines selected
#define IDM_EDIT_DELETE_FIRST_CHAR      4172  //Delete first char
#define IDM_EDIT_DELETE_FIRST_CHAR_MENU 4173  //Delete first char only if selection not empty
#define IDM_EDIT_DELETE_TRAILING_WHITESPACES 4174  //Delete trailing whitespaces
#define IDM_EDIT_UPPERCASE              4175  //Convert text to UPPERCASE
#define IDM_EDIT_LOWERCASE              4176  //Convert text to lowercase
#define IDM_EDIT_SENTENCECASE           4177  //Convert text to Sentence case.
#define IDM_EDIT_TITLECASE              4178  //Convert text to Title Case
#define IDM_EDIT_INVERTCASE             4179  //Convert text to iNVERT cASE
#define IDM_EDIT_NEWLINE_WIN            4180  //Windows new line format
#define IDM_EDIT_NEWLINE_UNIX           4181  //Unix new line format
#define IDM_EDIT_RECODE                 4182  //Recode dialog
#define IDM_VIEW_FONT                   4201  //Font dialog
#define IDM_VIEW_TEXT_COLOR             4202  //Text color dialog
#define IDM_VIEW_BG_COLOR               4203  //Background color dialog
#define IDM_VIEW_INCREASE_FONT          4204  //Increase font 1px
#define IDM_VIEW_DECREASE_FONT          4205  //Decrease font 1px
#define IDM_VIEW_ALIGN_LEFT             4206  //Left text align
#define IDM_VIEW_ALIGN_CENTER           4207  //Center text align
#define IDM_VIEW_ALIGN_RIGHT            4208  //Right text align
#define IDM_VIEW_WORDWRAP               4209  //Word wrap (on\off)
#define IDM_VIEW_ONTOP                  4210  //Always on top (on\off)
#define IDM_VIEW_SHOW_STATUSBAR         4211  //Show statusbar (on\off)
#define IDM_OPTIONS_EXEC                4251  //Execute command
#define IDM_OPTIONS_SAVETIME            4252  //Save file time (on\off)
#define IDM_OPTIONS_WATCHFILE           4253  //Watch file change (on\off)
#define IDM_OPTIONS_KEEPSPACE           4254  //Keep space (on\off)
#define IDM_OPTIONS_SINGLEOPEN_FILE     4255  //Single open file (on\off)
#define IDM_OPTIONS_SINGLEOPEN_PROGRAM  4256  //Single open program (on\off)
#define IDM_OPTIONS_SDI                 4257  //SDI mode
#define IDM_OPTIONS_MDI                 4258  //MDI mode
#define IDM_OPTIONS_PLUGINS             4259  //Plugins dialog
#define IDM_OPTIONS                     4260  //Options dialog
#define IDM_OPTIONS_READONLY            4261  //Read only (on\off)
#define IDM_WINDOW_TABVIEW_TOP          4301  //MDI tab window at the top of the main window
#define IDM_WINDOW_TABVIEW_BOTTOM       4302  //MDI tab window at the bottom of the main window
#define IDM_WINDOW_TABVIEW_NONE         4303  //Hide MDI tab
#define IDM_WINDOW_TABTYPE_STANDART     4304  //Standart MDI tab style
#define IDM_WINDOW_TABTYPE_BUTTONS      4305  //Buttons MDI tab style
#define IDM_WINDOW_TABTYPE_FLATBUTTONS  4306  //Flat buttons MDI tab style
#define IDM_WINDOW_TILEHORIZONTAL       4307  //MDI windows - horizontal tile
#define IDM_WINDOW_TILEVERTICAL         4308  //MDI windows - vertical tile
#define IDM_WINDOW_CASCADE              4309  //MDI windows - cascade
#define IDM_ABOUT                       4351  //About dialog
#define IDM_NONMENU_CHANGESIZE          4401  //Change style of the main window SW_RESTORE\SW_MAXIMIZE
#define IDM_NONMENU_DELLINE             4402  //Delete current line
#define IDM_NONMENU_PASTEANSI           4403  //Paste as ANSI text
#define IDM_NONMENU_MDINEXT             4404  //Activate next MDI window
#define IDM_NONMENU_MDIPREV             4405  //Activate previous MDI window
#define IDM_NONMENU_MDICLOSE            4406  //Close current MDI window
#define IDM_NONMENU_REOPEN_MSG          4407  //Show message that file is changed, if answer "Yes" reopen the file
#define IDM_NONMENU_REDETECT            4408  //Redetect code page of the current file
#define IDM_NONMENU_REOPENAS_ANSI       4409  //Reopen file as ANSI
#define IDM_NONMENU_REOPENAS_OEM        4410  //Reopen file as OEM
#define IDM_NONMENU_REOPENAS_UTF16LE    4411  //Reopen file as UTF16LE
#define IDM_NONMENU_REOPENAS_UTF16BE    4412  //Reopen file as UTF16BE
#define IDM_NONMENU_REOPENAS_UTF8       4413  //Reopen file as UTF8
#define IDM_NONMENU_REOPENAS_KOIR       4414  //Reopen file as KOI-R
#define IDM_NONMENU_SAVEAS_ANSI         4415  //Save file as ANSI
#define IDM_NONMENU_SAVEAS_OEM          4416  //Save file as OEM
#define IDM_NONMENU_SAVEAS_UTF16LE      4417  //Save file as UTF16LE
#define IDM_NONMENU_SAVEAS_UTF16BE      4418  //Save file as UTF16BE
#define IDM_NONMENU_SAVEAS_UTF8         4419  //Save file as UTF8
#define IDM_NONMENU_SAVEAS_KOIR         4420  //Save file as KOI-R
#define IDM_NONMENU_INSERTMODE          4421  //Insert mode (on\off)
#define IDM_RECENT_FILES                5001  //Delete dead recent files
                                              //5001 + n  open recent file n
#define IDM_LANGUAGE                    6001  //Activate internal language
                                              //6001 + n  activate language n
#define IDM_POPUP_OPENAS                7001  //7001 + n  open as popup menu item
#define IDM_POPUP_SAVEAS                8001  //8001 + n  save as popup menu item
                                              //
                                              //Example:
                                              //SendMessage(pd->hMainWnd, WM_COMMAND, IDM_FILE_NEW, 0);

//AkelPad WM_USER messages

#define AKDN_MAIN_ONSTART          (WM_USER + 1)
#define AKDN_MAIN_ONSTART_PRESHOW  (WM_USER + 2)
#define AKDN_MAIN_ONSTART_SHOW     (WM_USER + 3)
#define AKDN_MAIN_ONSTART_FINISH   (WM_USER + 4)
#define AKDN_MAIN_ONFINISH         (WM_USER + 5)
#define AKDN_EDIT_ONSTART          (WM_USER + 6)
#define AKDN_EDIT_ONFINISH         (WM_USER + 7)
#define AKDN_CONTEXTMENU           (WM_USER + 8)
#define AKDN_FRAME_NOWINDOWS       (WM_USER + 9)
#define AKDN_OPENDOCUMENT_START    (WM_USER + 10)
#define AKDN_OPENDOCUMENT_FINISH   (WM_USER + 11)
#define AKDN_SAVEDOCUMENT_START    (WM_USER + 12)
#define AKDN_SAVEDOCUMENT_FINISH   (WM_USER + 13)
#define AKDN_SIZE                  (WM_USER + 14)
#define AKDN_SEARCH_ENDED          (WM_USER + 15)
#define AKDN_ACTIVATE              (WM_USER + 16)
#define AKDN_RECENTFILESDELETE     (WM_USER + 17)
#define AKD_GETMAINPROC            (WM_USER + 101)
#define AKD_SETMAINPROC            (WM_USER + 102)
#define AKD_GETMAINPROCRET         (WM_USER + 103)
#define AKD_SETMAINPROCRET         (WM_USER + 104)
#define AKD_GETEDITPROC            (WM_USER + 105)
#define AKD_SETEDITPROC            (WM_USER + 106)
#define AKD_GETEDITPROCRET         (WM_USER + 107)
#define AKD_SETEDITPROCRET         (WM_USER + 108)
#define AKD_GETFRAMEPROC           (WM_USER + 109)
#define AKD_SETFRAMEPROC           (WM_USER + 110)
#define AKD_GETFRAMEPROCRET        (WM_USER + 111)
#define AKD_SETFRAMEPROCRET        (WM_USER + 112)
#define AKD_BEGINOPTIONS           (WM_USER + 113)
#define AKD_OPTION                 (WM_USER + 114)
#define AKD_ENDOPTIONS             (WM_USER + 115)
#define AKD_DLLCALL                (WM_USER + 116)
#define AKD_DLLUNLOAD              (WM_USER + 117)
#define AKD_DLLFIND                (WM_USER + 118)
#define AKD_DLLADD                 (WM_USER + 119)
#define AKD_DLLDELETE              (WM_USER + 120)
#define AKD_SAVEDOCUMENT           (WM_USER + 121)
#define AKD_GETTEXTLENGTH          (WM_USER + 122)
#define AKD_GETTEXTRANGE           (WM_USER + 123)
#define AKD_FREETEXT               (WM_USER + 124)
#define AKD_REPLACESELA            (WM_USER + 125)
#define AKD_REPLACESELW            (WM_USER + 126)
#define AKD_PASTE                  (WM_USER + 127)
#define AKD_COPY                   (WM_USER + 128)
#define AKD_TEXTFIND               (WM_USER + 129)
#define AKD_TEXTREPLACE            (WM_USER + 130)
#define AKD_GETEDITINFO            (WM_USER + 131)
#define AKD_GETFONT                (WM_USER + 132)
#define AKD_SETFONT                (WM_USER + 133)
#define AKD_SETMODIFY              (WM_USER + 134)
#define AKD_SETFILEPRINT           (WM_USER + 135)
#define AKD_SETMSGCREATE           (WM_USER + 136)
#define AKD_SETMSGBINARY           (WM_USER + 137)
#define AKD_GETQUEUE               (WM_USER + 138)
#define AKD_GETPRINTDLG            (WM_USER + 139)
#define AKD_GETPAGEDLG             (WM_USER + 140)
#define AKD_GETSELTEXTW            (WM_USER + 141)
#define AKD_GLOBALALLOC            (WM_USER + 142)
#define AKD_GLOBALLOCK             (WM_USER + 143)
#define AKD_GLOBALUNLOCK           (WM_USER + 144)
#define AKD_GLOBALFREE             (WM_USER + 145)
#define AKD_GETMODELESS            (WM_USER + 146)
#define AKD_SETMODELESS            (WM_USER + 147)
#define AKD_GETCODEPAGELIST        (WM_USER + 148)
#define AKD_RECODESEL              (WM_USER + 149)
#define AKD_SETNEWLINE             (WM_USER + 150)
#define AKD_GETRECENTFILES         (WM_USER + 151)

/*
AKDN_MAIN_ONSTART
_________________

Internal message.


AKDN_MAIN_ONSTART_PRESHOW
_________________________

Notification message, sends to the main procedure before showing main window.

wParam              == not used
(NMAINSHOW *)lParam == show main window characteristics, program show code is:
                         NMAINSHOW *nms=(NMAINSHOW *)lParam;

                         ShowWindow(hMainWnd, (nms->dwStyle == WS_MAXIMIZE)?SW_SHOWMAXIMIZED:SW_SHOW);
                         if (nms->dwShow != SW_SHOWNORMAL) ShowWindow(hMainWnd, nms->dwShow);

Return Value
 zero


AKDN_MAIN_ONSTART_SHOW
______________________

Notification message, sends to the main procedure when main window is shown.

wParam == not used
lParam == not used

Return Value
 zero


AKDN_MAIN_ONSTART_FINISH
________________________

Notification message, sends to the main procedure when main window is finished initialize.

wParam == not used
lParam == not used

Return Value
 zero


AKDN_MAIN_ONFINISH
__________________

Notification message, sends to the main procedure before destroying main window.

wParam == not used
lParam == not used

Return Value
 zero


AKDN_EDIT_ONSTART
_________________

Notification message, sends to the main procedure after rich edit window created.

(HWND)wParam == rich edit window
lParam       == not used

Return Value
 zero


AKDN_EDIT_ONFINISH
__________________

Notification message, sends to the main procedure before destroying edit window.

(HWND)wParam == rich edit window
lParam       == not used

Return Value
 zero


AKDN_CONTEXTMENU
________________

Notification message, sends to the main procedure before displaying context menu.

wParam                 == not used
(NCONTEXTMENU *)lParam == pointer to a NCONTEXTMENU structure

Return Value
 zero


AKDN_FRAME_NOWINDOWS
____________________

Notification message, sends to the main procedure when no frame windows in MDI client.

wParam == not used
lParam == not used

Return Value
 zero


AKDN_OPENDOCUMENT_START
_______________________

Notification message, sends to the main procedure before document opened.

(HWND)wParam            == rich edit window
(NOPENDOCUMENT *)lParam == pointer to a NOPENDOCUMENT structure
                           (NOPENDOCUMENTA *)lParam   if bOldWindows == TRUE
                           (NOPENDOCUMENTW *)lParam   if bOldWindows == FALSE

Return Value
 zero


AKDN_OPENDOCUMENT_FINISH
________________________

Notification message, sends to the main procedure after document opened.

(HWND)wParam == rich edit window
(int)lParam  == open document error. See EOD_* messages.

Return Value
 zero


AKDN_SAVEDOCUMENT_START
_______________________

Notification message, sends to the main procedure before document saved.

(HWND)wParam            == rich edit window
(NSAVEDOCUMENT *)lParam == pointer to a NSAVEDOCUMENT structure
                           (NSAVEDOCUMENTA *)lParam   if bOldWindows == TRUE
                           (NSAVEDOCUMENTW *)lParam   if bOldWindows == FALSE

Return Value
 zero


AKDN_SAVEDOCUMENT_FINISH
________________________

Notification message, sends to the main procedure after document saved.

(HWND)wParam == rich edit window
(int)lParam  == save document error. See ESD_* messages.

Return Value
 zero


AKDN_SIZE
_________

Notification message, sends to the main procedure before windows size changed.

(HWND)wParam    == rich edit window handle, if its size will be changed
                   NULL, if MDI client size will be changed
(RECT *)lParam  == pointer to a client RECT

Return Value
 zero


AKDN_SEARCH_ENDED
_________________

Notification message, sends to the main procedure after find/replace dialog found nothing.

(HWND)wParam == find/replace dialog
lParam       == not used

Return Value
 zero


AKDN_ACTIVATE
_____________

Notification message, sends to the main procedure after another instance of the program activates main window.

wParam == not used
lParam == not used

Return Value
 zero


AKDN_RECENTFILESDELETE
______________________

Notification message, sends to the main procedure before displaying message about deleted non-existent recent files records.

(int)wParam    == records deleted
(BOOL *)lParam == TRUE   show message
                  FALSE  don't show message

Return Value
 zero


AKD_GETMAINPROC, AKD_GETEDITPROC, AKD_GETFRAMEPROC
_______________  _______________  ________________

Get procedure from main window subclass chain.

(int)wParam            == index of procedure in chain, if positive search from top
                          if negative from beginning
(WNDPROCDATA **)lParam == procedure data

Return Value
 zero, if successfull

Example (get program main procedure data):
 WNDPROCDATA *wpd;
 SendMessage(pd->hMainWnd, AKD_GETMAINPROC, -1, (LPARAM)&wpd);


AKD_SETMAINPROC, AKD_SETEDITPROC, AKD_SETFRAMEPROC
_______________  _______________  ________________

Set procedure in main window subclass chain.

(WNDPROC)wParam        == procedure address,
                          if (wParam == NULL) then lParam must point to the procedure
                            data that will be removed from main window subclass chain.
(WNDPROCDATA **)lParam == procedure data,
                          if (lParam == NULL) then create new procedure data
                          if (*(WNDPROCDATA **)lParam == NULL) then create new
                            procedure data and set it on top of the main window
                            subclass chain.
                          if (*(WNDPROCDATA **)lParam != NULL) then set wParam
                            procedure in procedure data and update main window
                            subclass chain.

Return Value
 zero

Example:
 WNDPROCDATA *wpd=NULL;
 LRESULT CALLBACK NewMainProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
 {
   //...

   return wpd->NextProc(hWnd, uMsg, wParam, lParam);
 }
 wpd=NULL;
 SendMessage(pd->hMainWnd, AKD_SETMAINPROC, (WPARAM)NewMainProc, (LPARAM)&wpd);


AKD_GETMAINPROCRET, AKD_GETEDITPROCRET, AKD_GETFRAMEPROCRET
__________________  __________________  ___________________

Get return procedure from main window subclass chain. This procedure calls after message have been processed.

(int)wParam               == index of procedure in chain, if positive search from top
                             if negative from beginning
(WNDPROCRETDATA **)lParam == procedure data

Return Value
 zero, if successfull

Example:
 WNDPROCRETDATA *wprd;
 SendMessage(pd->hMainWnd, AKD_GETMAINPROC, 2, (LPARAM)&wprd);


AKD_SETMAINPROCRET, AKD_SETEDITPROCRET, AKD_SETFRAMEPROCRET
__________________  __________________  ___________________

Set return procedure in main window subclass chain. This procedure calls after message have been processed.

(WNDPROC)wParam        == procedure address,
                          if (wParam == NULL) then lParam must point to the procedure
                            data that will be removed from main window subclass chain.
(WNDPROCDATA **)lParam == procedure data,
                          if (lParam == NULL) then create new procedure data
                          if (*(WNDPROCDATA **)lParam == NULL) then create new
                            procedure data and set it on top of the main window
                            subclass chain.
                          if (*(WNDPROCDATA **)lParam != NULL) then set wParam
                            procedure in procedure data and update main window
                            subclass chain.

Return Value
 zero

Example:
 WNDPROCRETDATA *wprd=NULL;
 void CALLBACK NewMainProcRet(CWPRETSTRUCT *cwprs)
 {
   if (cwprs->message == WM_NOTIFY)
   {
     if (cwprs->wParam == ID_EDIT)
     {
       //...
     }
   }

   if (wprd->NextProc)
     wprd->NextProc(cwprs);
 }
 wprd=NULL;
 SendMessage(pd->hMainWnd, AKD_SETMAINPROCRET, (WPARAM)NewMainProcRet, (LPARAM)&wprd);


AKD_BEGINOPTIONS
________________

Begins read or save plugin options.

wParam                  == PO_READ  begins read options
                           PO_SAVE  begins save options
(unsigned char *)lParam == plugin name
                           (char *)lParam      if bOldWindows == TRUE
                           (wchar_t *)lParam   if bOldWindows == FALSE
Return Value
 HANDLE

Example read (bOldWindows == TRUE):
 PLUGINOPTIONA po;
 HANDLE hOptions;
 char szText[MAX_PATH];

 if (hOptions=(HANDLE)SendMessage(pd->hMainWnd, AKD_BEGINOPTIONS, PO_READ, (LPARAM)"ContextDLL"))
 {
   po.szOptionName="ContextMenuText";
   po.dwType=PO_STRING;
   po.lpData=(LPBYTE)szText;
   po.dwData=MAX_PATH;
   SendMessage(pd->hMainWnd, AKD_OPTION, (WPARAM)hOptions, (LPARAM)&po);

   SendMessage(pd->hMainWnd, AKD_ENDOPTIONS, (WPARAM)hOptions, 0);
 }

Example read (bOldWindows == FALSE):
 PLUGINOPTIONW po;
 HANDLE hOptions;
 wchar_t wszText[MAX_PATH];

 if (hOptions=(HANDLE)SendMessage(pd->hMainWnd, AKD_BEGINOPTIONS, PO_READ, (LPARAM)L"ContextDLL"))
 {
   po.wszOptionName=L"ContextMenuText";
   po.dwType=PO_STRING;
   po.lpData=(LPBYTE)wszText;
   po.dwData=MAX_PATH * sizeof(wchar_t);
   SendMessage(pd->hMainWnd, AKD_OPTION, (WPARAM)hOptions, (LPARAM)&po);

   SendMessage(pd->hMainWnd, AKD_ENDOPTIONS, (WPARAM)hOptions, 0);
 }

Example save (bOldWindows == TRUE):
 PLUGINOPTIONA po;
 HANDLE hOptions;
 char szText[MAX_PATH]="SomeText";

 if (hOptions=(HANDLE)SendMessage(pd->hMainWnd, AKD_BEGINOPTIONS, PO_SAVE, (LPARAM)"ContextDLL"))
 {
   po.szOptionName="ContextMenuText";
   po.dwType=PO_STRING;
   po.lpData=(LPBYTE)szText;
   po.dwData=lstrlen(szText) + 1;
   SendMessage(pd->hMainWnd, AKD_OPTION, (WPARAM)hOptions, (LPARAM)&po);

   SendMessage(pd->hMainWnd, AKD_ENDOPTIONS, (WPARAM)hOptions, 0);
 }

Example save (bOldWindows == FALSE):
 PLUGINOPTIONW po;
 HANDLE hOptions;
 wchar_t wszText[MAX_PATH]=L"SomeText";

 if (hOptions=(HANDLE)SendMessage(pd->hMainWnd, AKD_BEGINOPTIONS, PO_SAVE, (LPARAM)L"ContextDLL"))
 {
   po.wszOptionName=L"ContextMenuText";
   po.dwType=PO_STRING;
   po.lpData=(LPBYTE)wszText;
   po.dwData=lstrlenW(wszText) * sizeof(wchar_t) + 2;
   SendMessage(pd->hMainWnd, AKD_OPTION, (WPARAM)hOptions, (LPARAM)&po);

   SendMessage(pd->hMainWnd, AKD_ENDOPTIONS, (WPARAM)hOptions, 0);
 }


AKD_OPTION
__________

Read or save plugin option.

wParam                 == HANDLE returned by AKD_BEGINOPTIONS
(PLUGINOPTION *)lParam == pointer to a PLUGINOPTION structure
                          (PLUGINOPTIONA *)lParam   if bOldWindows == TRUE
                          (PLUGINOPTIONW *)lParam   if bOldWindows == FALSE
Return Value
 Length of the string copied to the buffer

Example:
 See AKD_BEGINOPTIONS examples


AKD_ENDOPTION
_____________

Ends plugin options.

wParam == HANDLE returned by AKD_BEGINOPTIONS
lParam == not used

Return Value
 TRUE   success
 FALSE  failed

Example:
 See AKD_BEGINOPTIONS examples


AKD_DLLCALL
___________

Call dll.

wParam == not used
lParam == pointer to a PLUGINCALLSEND structure if SendMessage used
          (PLUGINCALLSENDA *)lParam   if bOldWindows == TRUE
          (PLUGINCALLSENDW *)lParam   if bOldWindows == FALSE
          or pointer to a PLUGINCALLPOST, allocated with GlobalAlloc, if PostMessage used
          (PLUGINCALLPOSTA *)lParam   if bOldWindows == TRUE
          (PLUGINCALLPOSTW *)lParam   if bOldWindows == FALSE

Return Value
 EDL_FAILED                 operation failed
 EDL_UNLOADED               plugin unloaded
 EDL_NONUNLOADED_ACTIVE     plugin active
 EDL_NONUNLOADED_NONACTIVE  plugin non-active

Example SendMessage (bOldWindows == TRUE):
 PLUGINCALLSENDA pcs;
 pcs.pFunction="Plugin::Function";
 pcs.bOnStart=FALSE;
 pcs.lParam=0;
 SendMessage(pd->hMainWnd, AKD_DLLCALL, 0, (LPARAM)&pcs);

Example SendMessage (bOldWindows == FALSE):
 PLUGINCALLSENDW pcs;
 pcs.wpFunction=L"Plugin::Function";
 pcs.bOnStart=FALSE;
 pcs.lParam=0;
 SendMessage(pd->hMainWnd, AKD_DLLCALL, 0, (LPARAM)&pcs);

Example PostMessage (bOldWindows == TRUE):
 PLUGINCALLPOSTA *pcp;
 if (pcp=(PLUGINCALLPOSTA *)GlobalAlloc(GPTR, sizeof(PLUGINCALLPOSTA)))
 {
   lstrcpyn(pcp->szFunction, "Plugin::Function", MAX_PATH);
   pcp->bOnStart=FALSE;
   pcp->lParam=0;
   PostMessage(pd->hMainWnd, AKD_DLLCALL, 0, (LPARAM)pcp);
 }

Example PostMessage (bOldWindows == FALSE):
 PLUGINCALLPOSTW *pcp;
 if (pcp=(PLUGINCALLPOSTW *)GlobalAlloc(GPTR, sizeof(PLUGINCALLPOSTW)))
 {
   lstrcpynW(pcp->wszFunction, L"Plugin::Function", MAX_PATH);
   pcp->bOnStart=FALSE;
   pcp->lParam=0;
   PostMessage(pd->hMainWnd, AKD_DLLCALL, 0, (LPARAM)pcp);
 }


AKD_DLLUNLOAD
_____________

Unload dll.

wParam == not used
lParam == pointer to a PLUGINUNLOADSEND structure if SendMessage used
          (PLUGINUNLOADSENDA *)lParam   if bOldWindows == TRUE
          (PLUGINUNLOADSENDW *)lParam   if bOldWindows == FALSE
          or pointer to a PLUGINUNLOADPOST, allocated with GlobalAlloc, if PostMessage used
          (PLUGINUNLOADPOSTA *)lParam   if bOldWindows == TRUE
          (PLUGINUNLOADPOSTW *)lParam   if bOldWindows == FALSE

Return Value
 zero

Example SendMessage (bOldWindows == TRUE):
 PLUGINUNLOADSENDA pu;
 pu.szFunction=szFunction;
 pu.hInstanceDLL=pd->hInstanceDLL;
 pu.hThread=NULL;
 SendMessage(pd->hMainWnd, AKD_DLLUNLOAD, 0, (LPARAM)&pu);

Example SendMessage (bOldWindows == FALSE):
 PLUGINUNLOADSENDW pu;
 pu.wszFunction=wszFunction;
 pu.hInstanceDLL=pd->hInstanceDLL;
 pu.hThread=NULL;
 SendMessage(pd->hMainWnd, AKD_DLLUNLOAD, 0, (LPARAM)&pu);

Example PostMessage (bOldWindows == TRUE):
 PLUGINUNLOADPOSTA *pu;
 if (pu=(PLUGINUNLOADPOSTA *)GlobalAlloc(GPTR, sizeof(PLUGINUNLOADPOSTA)))
 {
   lstrcpyn(pu->szFunction, szFunction, MAX_PATH);
   pu->hInstanceDLL=pd->hInstanceDLL;
   pu->hThread=NULL;
   PostMessage(pd->hMainWnd, AKD_DLLUNLOAD, 0, (LPARAM)pu);
 }

Example PostMessage (bOldWindows == FALSE):
 PLUGINUNLOADPOSTW *pu;
 if (pu=(PLUGINUNLOADPOSTW *)GlobalAlloc(GPTR, sizeof(PLUGINUNLOADPOSTW)))
 {
   lstrcpynW(pu->wszFunction, wszFunction, MAX_PATH);
   pu->hInstanceDLL=pd->hInstanceDLL;
   pu->hThread=NULL;
   PostMessage(pd->hMainWnd, AKD_DLLUNLOAD, 0, (LPARAM)pu);
 }


AKD_DLLFIND
___________

Get dll stack function structure pointer.

(unsigned char *)wParam == function name, format "Plugin::Function"
                           (char *)wParam      if bOldWindows == TRUE
                           (wchar_t *)wParam   if bOldWindows == FALSE
lParam                  == not used

Return Value
 pointer to a PLUGINFUNCTION structure

Example (bOldWindows == TRUE):
 PLUGINFUNCTIONA *pf;
 pf=(PLUGINFUNCTIONA *)SendMessage(pd->hMainWnd, AKD_DLLFIND, (WPARAM)"SomePlugin::SomeFunction", 0);
 if (pf) pf->bRunning=FALSE; //change active status

Example (bOldWindows == FALSE):
 PLUGINFUNCTIONW *pf;
 pf=(PLUGINFUNCTIONW *)SendMessage(pd->hMainWnd, AKD_DLLFIND, (WPARAM)L"SomePlugin::SomeFunction", 0);
 if (pf) pf->bRunning=FALSE;  //change active status


AKD_DLLADD
__________

Add dll stack function structure.

wParam                   == not used
(PLUGINFUNCTION *)lParam == function structure pointer
                            (PLUGINFUNCTIONA *)lParam   if bOldWindows == TRUE
                            (PLUGINFUNCTIONW *)lParam   if bOldWindows == FALSE

Return Value
 pointer to a PLUGINFUNCTION structure in stack

Example add plugin hotkey (bOldWindows == TRUE):
 void CALLBACK PluginProc(void *lpParameter)
 {
 }
 PLUGINFUNCTIONA pf;
 pf.szFunction[0]='\0';
 pf.nFunctionLen=0;
 pf.wHotkey=589;       //Ctrl+M
 pf.bOnStart=FALSE;
 pf.bRunning=FALSE;
 pf.PluginProc=(PLUGINPROC)PluginProc;
 pf.lpParameter=NULL;
 SendMessage(pd->hMainWnd, AKD_DLLADD, 0, (LPARAM)&pf);

Example add plugin hotkey (bOldWindows == FALSE):
 void CALLBACK PluginProc(void *lpParameter)
 {
 }
 PLUGINFUNCTIONW pf;
 pf.wszFunction[0]='\0';
 pf.nFunctionLen=0;
 pf.wHotkey=589;       //Ctrl+M
 pf.bOnStart=FALSE;
 pf.bRunning=FALSE;
 pf.PluginProc=(PLUGINPROC)PluginProc;
 pf.lpParameter=NULL;
 SendMessage(pd->hMainWnd, AKD_DLLADD, 0, (LPARAM)&pf);


AKD_DLLDELETE
_____________

Delete dll stack function structure.

wParam                   == not used
(PLUGINFUNCTION *)lParam == pointer to a PLUGINFUNCTION structure
                            (PLUGINFUNCTIONA *)lParam   if bOldWindows == TRUE
                            (PLUGINFUNCTIONW *)lParam   if bOldWindows == FALSE

Return Value
 zero

Example (bOldWindows == TRUE):
 PLUGINFUNCTIONA *pf;
 if (pf=(PLUGINFUNCTIONA *)SendMessage(pd->hMainWnd, AKD_DLLFIND, (WPARAM)"SomePlugin::SomeFunction", 0))
 {
   SendMessage(pd->hMainWnd, AKD_DLLDELETE, 0, (LPARAM)pf);
   pf=NULL;
 }

Example (bOldWindows == FALSE):
 PLUGINFUNCTIONW *pf;
 if (pf=(PLUGINFUNCTIONW *)SendMessage(pd->hMainWnd, AKD_DLLFIND, (WPARAM)L"SomePlugin::SomeFunction", 0))
 {
   SendMessage(pd->hMainWnd, AKD_DLLDELETE, 0, (LPARAM)pf);
   pf=NULL;
 }


AKD_SAVEDOCUMENT
________________

Save file.

(HWND)wParam           == rich edit window
(SAVEDOCUMENT *)lParam == pointer to a SAVEDOCUMENT structure
                         (SAVEDOCUMENTA *)lParam   if bOldWindows == TRUE
                         (SAVEDOCUMENTW *)lParam   if bOldWindows == FALSE

Return Value
 zero                success
 ESD_OPEN            can't open file
 ESD_WRITE           can't write to file
 ESD_CODEPAGE_ERROR  code page isn't implemented
 ESD_STOP            stopped from AKDN_SAVEDOCUMENT_START
 ESD_STREAMOUT       error in EM_STREAMOUT

Example (bOldWindows == TRUE):
 SAVEDOCUMENTA sd;

 lstrcpyn(sd.szFile, "C:\\MyFile.txt", MAX_PATH);
 sd.nCodePage=65001;
 sd.bBOM=TRUE;
 sd.bActivate=TRUE;
 SendMessage(pd->hMainWnd, AKD_SAVEDOCUMENT, (WPARAM)pd->hWndEdit, (LPARAM)&sd);

Example (bOldWindows == FALSE):
 SAVEDOCUMENTW sd;

 lstrcpynW(sd.wszFile, L"C:\\MyFile.txt", MAX_PATH);
 sd.nCodePage=65001;
 sd.bBOM=TRUE;
 sd.bActivate=TRUE;
 SendMessage(pd->hMainWnd, AKD_SAVEDOCUMENT, (WPARAM)pd->hWndEdit, (LPARAM)&sd);


AKD_GETTEXTLENGTH
_________________

Get rich edit window text length.

(HWND)wParam == rich edit window
lParam       == not used

Return Value
 text length

Example:
 int nLength=SendMessage(pd->hMainWnd, AKD_GETTEXTLENGTH, (WPARAM)pd->hWndEdit, 0);


AKD_GETSELTEXTW
_______________

Retrieves the currently selected text in a rich edit control.

(HWND)wParam == rich edit window
lParam       == not used

Return Value
 text pointer

Example:
 wchar_t *wpText;

 wpText=SendMessage(pd->hMainWnd, AKD_GETSELTEXTW, (WPARAM)pd->hWndEdit, 0);
 SendMessage(pd->hMainWnd, AKD_FREETEXT, 0, (LPARAM)wpText);


AKD_GETTEXTRANGE
________________

Retrieves a specified range of characters from a rich edit control.

(HWND)wParam           == rich edit window
(GETTEXTRANGE *)lParam == pointer to a GETTEXTRANGE structure

Return Value
 Text length in TCHARs without null character

Example (bOldWindows == TRUE):
 GETTEXTRANGE gtr;

 gtr.cpMin=0;
 gtr.cpMax=-1;
 SendMessage(pd->hMainWnd, AKD_GETTEXTRANGE, (WPARAM)pd->hWndEdit, (LPARAM)&gtr);
 MessageBox(pd->hMainWnd, (char *)gtr.pText, "Test", MB_OK);
 SendMessage(pd->hMainWnd, AKD_FREETEXT, 0, (LPARAM)gtr.pText);

Example (bOldWindows == FALSE):
 GETTEXTRANGE gtr;

 gtr.cpMin=0;
 gtr.cpMax=-1;
 SendMessage(pd->hMainWnd, AKD_GETTEXTRANGE, (WPARAM)pd->hWndEdit, (LPARAM)&gtr);
 MessageBoxW(pd->hMainWnd, (wchar_t *)gtr.pText, L"Test", MB_OK);
 SendMessage(pd->hMainWnd, AKD_FREETEXT, 0, (LPARAM)gtr.pText);


AKD_FREETEXT
____________

Free text buffer allocated with AKD_GETTEXTRANGE.

wParam                  == not used
(unsigned char *)lParam == buffer pointer

Return Value
 TRUE   success
 FALSE  failed

See AKD_GETTEXTRANGE.


AKD_REPLACESELA
_______________

Replace selection of the rich edit control.

(HWND)wParam   == rich edit window
(char *)lParam == text pointer

Return Value
 zero

Example:
 SendMessage(pd->hMainWnd, AKD_REPLACESELA, (WPARAM)pd->hWndEdit, (LPARAM)"SomeString");


AKD_REPLACESELW
_______________

Replace selection of the rich edit control.

(HWND)wParam      == rich edit window
(wchar_t *)lParam == text pointer

Return Value
 zero

Example:
 SendMessage(pd->hMainWnd, AKD_REPLACESELW, (WPARAM)pd->hWndEdit, (LPARAM)L"SomeString");


AKD_PASTE
_________

Paste text from clipboard to the rich edit control.

(HWND)wParam == rich edit window
(BOOL)lParam == FALSE paste as Unicode text (default)
                TRUE  paste as ANSI text

Return Value
 zero

Example:
 SendMessage(pd->hMainWnd, AKD_PASTE, (WPARAM)pd->hWndEdit, FALSE);


AKD_COPY
________

Copy text to clipboard from rich edit control.

(HWND)wParam == rich edit window
lParam       == not used

Return Value
 zero

Example:
 SendMessage(pd->hMainWnd, AKD_COPY, (WPARAM)pd->hWndEdit, 0);


AKD_TEXTFIND
____________

Finds text in a rich edit control.

(HWND)wParam       == rich edit window
(TEXTFIND *)lParam == pointer to a TEXTFIND structure
                      (TEXTFINDA *)lParam   if bOldWindows == TRUE
                      (TEXTFINDW *)lParam   if bOldWindows == FALSE
Return Value
 Character position of the next match. If there are no more matches, the return value is 1.

Example (bOldWindows == TRUE):
 TEXTFINDA tf;

 tf.dwFlags=FR_DOWN|FR_BEGINNING|FR_MATCHCASE;
 tf.pFindIt="Text to find";
 SendMessage(pd->hMainWnd, AKD_TEXTFIND, (WPARAM)pd->hWndEdit, (LPARAM)&tf);

Example (bOldWindows == FALSE):
 TEXTFINDW tf;

 tf.dwFlags=FR_DOWN|FR_BEGINNING|FR_MATCHCASE;
 tf.wpFindIt=L"Text to find";
 SendMessage(pd->hMainWnd, AKD_TEXTFIND, (WPARAM)pd->hWndEdit, (LPARAM)&tf);


AKD_TEXTREPLACE
_______________

Replaces text in a rich edit control.

(HWND)wParam          == rich edit window
(TEXTREPLACE *)lParam == pointer to a TEXTREPLACE structure
                         (TEXTREPLACEA *)lParam   if bOldWindows == TRUE
                         (TEXTREPLACEW *)lParam   if bOldWindows == FALSE
Return Value
 Character position of the next match. If there are no more matches, the return value is 1.

Example (bOldWindows == TRUE):
 TEXTREPLACEA tr;

 tr.dwFlags=FR_DOWN|FR_BEGINNING|FR_MATCHCASE;
 tr.pFindIt="Text to find";
 tr.pReplaceWith="Text to replace";
 tr.bAll=TRUE;
 SendMessage(pd->hMainWnd, AKD_TEXTREPLACE, (WPARAM)pd->hWndEdit, (LPARAM)&tr);

Example (bOldWindows == FALSE):
 TEXTREPLACEW tr;

 tr.dwFlags=FR_DOWN|FR_BEGINNING|FR_MATCHCASE;
 tr.wpFindIt=L"Text to find";
 tr.wpReplaceWith=L"Text to replace";
 tr.bAll=TRUE;
 SendMessage(pd->hMainWnd, AKD_TEXTREPLACE, (WPARAM)pd->hWndEdit, (LPARAM)&tr);


AKD_RECODESEL
_____________

Recode text in a rich edit control.

(HWND)wParam         == rich edit window
(TEXTRECODE *)lParam == pointer to a TEXTRECODE structure

Return Value
 zero

Example:
 TEXTRECODE tr;

 tr.nCodePageFrom=1251;
 tr.nCodePageTo=866;
 SendMessage(pd->hMainWnd, AKD_RECODESEL, (WPARAM)pd->hWndEdit, (LPARAM)&tr);


AKD_GETEDITINFO
_______________

Get rich edit window info.

(HWND)wParam       == rich edit window,
                      NULL for current edit window
(EDITINFO *)lParam == pointer to a EDITINFO structure

Return Value
 TRUE   success
 FALSE  failed

Example:
 EDITINFO ei;

 SendMessage(pd->hMainWnd, AKD_GETEDITINFO, (WPARAM)NULL, (LPARAM)&ei);


AKD_GETFONT
___________

Get font.

(HWND)wParam == rich edit window,
                NULL for current edit window
lParam       == not used

Return Value
 pointer to a LOGFONT structure

Example (bOldWindows == TRUE):
 LOGFONT *lf=(LOGFONT *)SendMessage(pd->hMainWnd, AKD_GETFONT, (WPARAM)NULL, 0);

Example (bOldWindows == FALSE):
 LOGFONTW *lf=(LOGFONTW *)SendMessage(pd->hMainWnd, AKD_GETFONT, (WPARAM)NULL, 0);


AKD_SETFONT
___________

Set font.

(HWND)wParam      == rich edit window
(LOGFONT *)lParam == pointer to a LOGFONT structure

Return Value
 zero

Example (bOldWindows == TRUE):
 LOGFONT *lf;

 lf=(LOGFONT *)SendMessage(pd->hMainWnd, AKD_GETFONT, 0, 0);
 lf->lfHeight-=2;
 lstrcpy(lf->lfFaceName, "Courier New");
 SendMessage(pd->hMainWnd, AKD_SETFONT, (WPARAM)pd->hWndEdit, (LPARAM)lf);

Example (bOldWindows == FALSE):
 LOGFONTW *lf;

 lf=(LOGFONTW *)SendMessage(pd->hMainWnd, AKD_GETFONT, 0, 0);
 lf->lfHeight-=2;
 lstrcpyW(lf->lfFaceName, L"Courier New");
 SendMessage(pd->hMainWnd, AKD_SETFONT, (WPARAM)pd->hWndEdit, (LPARAM)lf);


AKD_SETMODIFY
_____________

Set rich edit window modification flag.

(HWND)wParam == rich edit window,
                NULL for current edit window
(BOOL)lParam == TRUE  contents of edit control have been modified
                FALSE contents of edit control haven't been modified

Return Value
 zero

Example:
 SendMessage(pd->hMainWnd, AKD_SETMODIFY, (WPARAM)pd->hWndEdit, TRUE);


AKD_SETNEWLINE
______________

Set rich edit window new line format.

(HWND)wParam == rich edit window,
                NULL for current edit window
(int)lParam  == NEWLINE_WIN  Windows/DOS new line format (\r\n)
                NEWLINE_UNIX Unix new line format (\n)

Return Value
 zero

Example:
 SendMessage(pd->hMainWnd, AKD_SETNEWLINE, (WPARAM)pd->hWndEdit, NEWLINE_UNIX);


AKD_SETFILEPRINT
________________

Next opened file will be printed.

(BOOL)wParam == TRUE  will be printed
                FALSE will not be printed
lParam       == not used

Return Value
 zero

Example:
 SendMessage(pd->hMainWnd, AKD_SETFILEPRINT, TRUE, 0);


AKD_SETMSGCREATE
________________

Create unexisted file autoanswer.

(int)wParam == AUTOANSWER_ASK  Show message (default)
               AUTOANSWER_YES  Create unexisted file
               AUTOANSWER_NO   Don't create unexisted file
lParam      == not used

Return Value
 zero

Example:
 SendMessage(pd->hMainWnd, AKD_SETMSGCREATE, AUTOANSWER_YES, 0);


AKD_SETMSGBINARY
________________

Open binary file autoanswer.

(int)wParam == AUTOANSWER_ASK  Show message (default)
               AUTOANSWER_YES  Open binary file
               AUTOANSWER_NO   Don't open binary file
lParam      == not used

Return Value
 zero

Example:
 SendMessage(pd->hMainWnd, AKD_SETMSGBINARY, AUTOANSWER_YES, 0);


AKD_GETCODEPAGELIST
___________________

Get code page list.

wParam == not used
lParam == not used

Return Value
 pointer to a code page array, in which last element is zero

Example:
 int *lpCodepageList=(int *)SendMessage(pd->hMainWnd, AKD_GETCODEPAGELIST, 0, 0);


AKD_GETRECENTFILES
__________________

Get recent files info.

wParam == not used
(RECENTFILES *)lParam == pointer to a RECENTFILES structure
                         (RECENTFILESA *)lParam   if bOldWindows == TRUE
                         (RECENTFILESW *)lParam   if bOldWindows == FALSE

Return Value
 zero

Example (bOldWindows == TRUE):
 RECENTFILESA rf;

 SendMessage(pd->hMainWnd, AKD_GETRECENTFILES, 0, (LPARAM)&rf);

Example (bOldWindows == FALSE):
 RECENTFILESW rf;

 SendMessage(pd->hMainWnd, AKD_GETRECENTFILES, 0, (LPARAM)&rf);


AKD_GETQUEUE
____________

Indicates the type of messages found in the main thread's message queue
(see description for GetQueueStatus in MSDN).

wParam == specifies the types of messages for which to check
lParam == not used

Return Value
  number of messages

Example:
 DWORD dwStatus=SendMessage(pd->hMainWnd, AKD_GETQUEUE, QS_ALLEVENTS, 0);


AKD_GETPRINTDLG
_______________

Get print dialog settings.

wParam == not used
lParam == not used

Return Value
 pointer to a PRINTDLG structure

Example (bOldWindows == TRUE):
 PRINTDLG *pdg=(PRINTDLG *)SendMessage(pd->hMainWnd, AKD_GETPRINTDLG, 0, 0);

Example (bOldWindows == FALSE):
 PRINTDLGW *pdg=(PRINTDLGW *)SendMessage(pd->hMainWnd, AKD_GETPRINTDLG, 0, 0);


AKD_GETPAGEDLG
______________

Get page dialog settings.

wParam == not used
lParam == not used

Return Value
 pointer to a PAGESETUPDLG structure

Example (bOldWindows == TRUE):
 PAGESETUPDLG *psdg=(PAGESETUPDLG *)SendMessage(pd->hMainWnd, AKD_GETPAGEDLG, 0, 0);

Example (bOldWindows == FALSE):
 PAGESETUPDLGW *psdg=(PAGESETUPDLGW *)SendMessage(pd->hMainWnd, AKD_GETPAGEDLG, 0, 0);


AKD_GLOBALALLOC
_______________

Allocates the specified number of bytes from the heap (see description for GlobalAlloc in MSDN).

wParam == memory allocation attributes
lParam == number of bytes to allocate

Return Value
  handle to the newly allocated memory object

Example:
 HGLOBAL hMem=(HGLOBAL)SendMessage(pd->hMainWnd, AKD_GLOBALALLOC, GHND, 128);


AKD_GLOBALLOCK
______________

Locks a global memory object (see description for GlobalLock in MSDN).

wParam == handle to the global memory object
lParam == not used

Return Value
  pointer to the first byte of the memory block

Example:
 void *pMem=(void *)SendMessage(pd->hMainWnd, AKD_GLOBALLOCK, (WPARAM)hMem, 0);


AKD_GLOBALUNLOCK
________________

Decrements the lock count associated with a memory object (see description for GlobalUnlock in MSDN).

wParam == handle to the global memory object
lParam == not used

Return Value
 unlock result

Example:
 BOOL bUnlock=SendMessage(pd->hMainWnd, AKD_GLOBALUNLOCK, (WPARAM)hMem, 0);


AKD_GLOBALFREE
______________

Frees the specified global memory object (see description for GlobalFree in MSDN).

wParam == handle to the global memory object
lParam == not used

Return Value
 free result

Example:
 HGLOBAL hMem=SendMessage(pd->hMainWnd, AKD_GLOBALFREE, (WPARAM)hMem, 0);


AKD_GETMODELESS
_______________

Get modeless dialog handle.

wParam == not used
lParam == not used

Return Value
 dialog handle

Example:
 HWND hDlg=(HWND)SendMessage(pd->hMainWnd, AKD_GETMODELESS, 0, 0);


AKD_SETMODELESS
_______________

Set modeless dialog handle.

(HWND)wParam == dialog handle
lParam       == not used

Return Value
 zero

Example:
 SendMessage(pd->hMainWnd, AKD_SETMODELESS, (LPARAM)hMyDialog, 0);
*/


//AkelPad WM_COPYDATA messages

#define CD_OPENDOCUMENT   1


/*
CD_OPENDOCUMENT
_______________

Open file.

(DWORD)dwData          == CD_OPENDOCUMENT
(DWORD)cbData          == sizeof(OPENDOCUMENTA)    if bOldWindows == TRUE
                          sizeof(OPENDOCUMENTW)    if bOldWindows == FALSE
(OPENDOCUMENT *)lpData == (OPENDOCUMENTA *)lpData  if bOldWindows == TRUE
                          (OPENDOCUMENTW *)lpData  if bOldWindows == FALSE

Return Value
 zero                success
 EOD_ADT_OPEN        autodetect codepage, can't open file
 EOD_ADT_ALLOC       autodetect codepage, can't allocate buffer
 EOD_ADT_READ        autodetect codepage, read file error
 EOD_ADT_BINARY      autodetect codepage, file is binary
 EOD_OPEN            can't open file
 EOD_CANCEL          user press cancel
 EOD_WINDOW_EXIST    file already opened
 EOD_CODEPAGE_ERROR  code page isn't implemented
 EOD_STOP            stopped from AKDN_OPENDOCUMENT_START
 EOD_STREAMIN        error in EM_STREAMIN

Example (bOldWindows == TRUE):
  OPENDOCUMENTA od;
  COPYDATASTRUCT cds;

  lstrcpyn(od.szFile, "C:\\MyFile.txt", MAX_PATH);
  od.szWorkDir[0]='\0';
  od.dwFlags=OD_ADT_BINARY_ERROR|OD_ADT_REG_CODEPAGE;
  od.nCodePage=0;
  od.bBOM=0;

  cds.dwData=CD_OPENDOCUMENT;
  cds.cbData=sizeof(OPENDOCUMENTA);
  cds.lpData=&od;
  SendMessage(pd->hMainWnd, WM_COPYDATA, (WPARAM)pd->hMainWnd, (LPARAM)&cds);

Example (bOldWindows == FALSE):
  OPENDOCUMENTW od;
  COPYDATASTRUCT cds;

  lstrcpynW(od.wszFile, L"C:\\MyFile.txt", MAX_PATH);
  od.wszWorkDir[0]='\0';
  od.dwFlags=OD_ADT_BINARY_ERROR|OD_ADT_REG_CODEPAGE;
  od.nCodePage=0;
  od.bBOM=0;

  cds.dwData=CD_OPENDOCUMENT;
  cds.cbData=sizeof(OPENDOCUMENTW);
  cds.lpData=&od;
  SendMessage(pd->hMainWnd, WM_COPYDATA, (WPARAM)pd->hMainWnd, (LPARAM)&cds);
*/
